/*
Written by: George Ng
ngg@ecf.utoronto.ca
Test demonstrates the functionality of the ZBT RAM controller provided by Xilinx
ADDR_BUS_INTERFACE.v 
CTRL_BUS_INTERFACE.v
DATA_BUS_INTERFACE.v
PIPELINES.v
ZBT_CONTROL.v
Testbench performs reads and writes to the RAM behaviourial model
*/

`timescale  1 ns / 1 ps
`define ADDR_BITS 19
`define DATA_BITS 32

module ZBT_tb();

   `define  sys_clk_per 37.037

   reg sys_clk;

   //external ram wires are ALL CAPS
   //data bus wires
   wire [(`DATA_BITS-1):0] RAM_DATA_BUS;

   wire [(`ADDR_BITS-1):0] RAM_ADDR;      // ADDR GOES OUT TO THE RAM
   wire RAM_LBO;        // BURST MODE (0 = LINEAR, 1 = INTERLEAVED)
   wire RAM_CLOCK;      // CLOCK TO THE RAM
   wire RAM_CLOCKEN_N;  // SYNCHRONOUS CLOCK ENABLE
   wire RAM_ADV_LD;     // SYNCHRONOUS ADDRESS ADV/LD
   wire RAM_WEA_N;      // SYNCHRONOUS BYTE WRITE ENABLE A
   wire RAM_WEB_N;      // SYNCHRONOUS BYTE WRITE ENABLE B
   wire RAM_WEC_N;      // SYNCHRONOUS BYTE WRITE ENABLE C
   wire RAM_WED_N;      // SYNCHRONOUS BYTE WRITE ENABLE D
   wire RAM_WE_N;       // READ WRITE CONTROL GOES OUT TO THE RAM
   wire RAM_OE_N;       // OUTPUT ENABLE
   wire RAM_CE0_N;      // SYNCHRONOUS CHIP ENABLE
   wire RAM_CE1;        // SYNCHRONOUS CHIP ENABLE
   wire RAM_CE2_N;      // SYNCHRONOUS CHIP ENABLE
   wire RAM_ZZ;         // SNOOZE MODE

   //test bench signals
   reg [(`ADDR_BITS-1):0] user_ram_addr;
   reg [(`DATA_BITS-1):0] user_write_data;
   reg user_read_writez;   
   wire [(`DATA_BITS-1):0]  user_read_data;  //data output from controller
   reg user_clocken;
   reg user_oe;
   reg user_ce0;
   reg user_ce1;
   reg user_ce2;
   reg user_adv_ld;
   reg user_wea;
   reg user_web;
   reg user_wec;
   reg user_wed;
   reg user_lbo;
   reg user_zz;
   
   //when signal is _N it means that the value in should be driven to 0 for it 
   //to be active

   initial begin
      #(`sys_clk_per/2) //realign signals to +ve clock edge
     
      // ram settings
      user_clocken = 1'b1; //always enable clock to ZBT
      user_read_writez = 1'b1; //1'b1 is a read, 1'b0 is a write
      user_oe = 1'b1; //set according to spec sheet for reads
      user_ce0 = 1'b1; //set according to spec sheet for reads 1'b1
      user_ce1 = 1'b1; //set according to spec sheet for reads 1'b1
      user_ce2 = 1'b1; //set according to spec sheet for reads 1'b1
      user_adv_ld = 1'b1; //set according to spec sheet to load addresses
      user_wea = 1'b1;
      user_web = 1'b1;
      user_wec = 1'b1;
      user_wed = 1'b1;
      user_lbo = 1'b1; //don't use linear burst 
      user_zz = 1'b0; //RAM don't sleep

      #(10*`sys_clk_per); //wait for some time before starting testbench
      // peform some memory reads
      user_ram_addr = 19'h0;
      #(`sys_clk_per);
      user_ram_addr = 19'h1;
      #(`sys_clk_per);
      user_ram_addr = 19'h2;
      #(`sys_clk_per);
      user_ram_addr = 19'h3;
      #(`sys_clk_per);      
      user_ram_addr = 19'h4;
      #(`sys_clk_per);      
      user_ram_addr = 19'h5;
      #(`sys_clk_per);      
      //begin writes
      user_read_writez = 1'b0;
      user_write_data = 32'hAAAA_BBBB;
      user_ram_addr = 19'h0;
      #(`sys_clk_per);     
      user_ram_addr = 19'h1;
      user_write_data = 32'hCCCC_DDDD;      
      #(`sys_clk_per);  
      user_ram_addr = 19'h2;
      user_write_data = 32'hEEEE_FFFF;      
      #(`sys_clk_per);  
      //read from just written addresses
      user_read_writez = 1'b1;
      user_ram_addr = 19'h0;
      #(`sys_clk_per);
      user_ram_addr = 19'h1;
      #(`sys_clk_per);
      user_ram_addr = 19'h2;
      #(`sys_clk_per);

   end

   // generate clock
   initial begin
      sys_clk = 0;
      forever #(`sys_clk_per/2) sys_clk = ~sys_clk;
   end



   ZBT_CONTROL uZBT_CONTROL (
      .ram_clock(RAM_CLOCK), //output clock from controller to RAM
      .ram_clocken_n(RAM_CLOCKEN_N),
      .ram_we_n(RAM_WE_N),
      .ram_oe_n(RAM_OE_N),
      .ram_ce0_n(RAM_CE0_N),
      .ram_ce1(RAM_CE1),
      .ram_ce2_n(RAM_CE2_N),
      .ram_adv_ld(RAM_ADV_LD),
      .ram_wea_n(RAM_WEA_N),
      .ram_web_n(RAM_WEB_N),
      .ram_wec_n(RAM_WEC_N),
      .ram_wed_n(RAM_WED_N),
      .ram_lbo(RAM_LBO),
      .ram_zz(RAM_ZZ),
      .ram_addr(RAM_ADDR),
      .ram_data_bus(RAM_DATA_BUS),

      //user signals
      .user_read_data(user_read_data),
      .user_ram_clock(sys_clk), //connect the RAM to system clock
      .user_clocken(user_clocken),
      .user_read_writez(user_read_writez),
      .user_oe(user_oe),
      .user_ce0(user_ce0),
      .user_ce1(user_ce1),
      .user_ce2(user_ce2),
      .user_adv_ld(user_adv_ld),
      .user_wea(user_wea),
      .user_web(user_web),
      .user_wec(user_wec),
      .user_wed(user_wed),
      .user_lbo(user_lbo),
      .user_zz(user_zz),
      .clock(sys_clk), //connect controller logic to system clock
      .user_write_data(user_write_data),
      .user_ram_addr(user_ram_addr)
   );

   // ZBT RAM model
   K7N163601A uK7N163601A(
      .Dq(RAM_DATA_BUS), 
      .Addr(RAM_ADDR), 
      .K(RAM_CLOCK), 
      .CKEb(RAM_CLOCKEN_N),
      .Bwa_n(RAM_WEA_N),
      .Bwb_n(RAM_WEB_N),
      .Bwc_n(RAM_WEC_N),
      .Bwd_n(RAM_WED_N),
      .WEb(RAM_WE_N),
      .ADV(RAM_ADV_LD),
      .OEb(RAM_OE_N),
      .CS1b(RAM_CE0_N),
      .CS2(RAM_CE1),
      .CS2b(RAM_CE2_N),
      .LBOb(RAM_LBO),
      .ZZ(RAM_ZZ)
   );

endmodule