// Written by George Ng 
// ngg@ecf.utoronto.ca
// March 23, 2005
// Module manages the transfer of data from the ZBT Memory to a FIFO
// Data is burst across ZBT BUS until FIFO is full using a counter to generate addresses
// FSM deals with the initial 4 clock latency of the ZBT
// Also generates the last_data signal for case when last pixel data does not fill the FIFO
// (for with by the fifo2disp_ctrl controller)

`timescale  1 ns / 1 ps

module mem2fifo_ctrl (
   clk,
   reset,
   full,
   empty,
   mem_addr,
   wr_en,
   last_data
   );
   
   input  clk;
   input  reset;
   input  full;
   input  empty;
   output [18:0] mem_addr;
   output wr_en;
   output last_data;
   
   wire clk;
   wire reset;
   reg  mem_addr_ctr_en;
   reg  wr_en;
   reg  [18:0] mem_addr;
   reg  last_data;

   reg [1:0] y, Y; //FSM state vector
   reg [2:0] mem_delay_ctr; //memory read data valid delay counter
   
   parameter [1:0] IDLE = 2'b00, MEMREAD = 2'b01, EN_WR2FIFO = 2'b10;
   
   // Mealy Type FSM
   // Define the next state and output combinational circuit
   always@(y or empty or full or mem_delay_ctr or last_data)
      case(y)
         IDLE       :      if(empty) begin
                              mem_addr_ctr_en = 1'b1;
                              wr_en = 1'b0;
                              Y = MEMREAD;
                           end
                           else begin
                              mem_addr_ctr_en = 1'b0;
                              wr_en = 1'b0;
                              Y = IDLE;
                           end
         MEMREAD    :      if(mem_delay_ctr == 4) begin
                              mem_addr_ctr_en = 1'b1;
                              wr_en = 1'b1;
                              Y = EN_WR2FIFO;
                           end
                           else begin
                              mem_addr_ctr_en = 1'b1;
                              wr_en = 1'b0;
                              Y = MEMREAD;
                           end
         EN_WR2FIFO :      if(full || last_data) begin
                              mem_addr_ctr_en = 1'b0;
                              wr_en = 1'b0;
                              Y = IDLE;
                           end
                           else begin
                              mem_addr_ctr_en = 1'b1;
                              wr_en = 1'b1;
                              Y = EN_WR2FIFO;
                           end
      endcase

   //Define sequential block
   always@(posedge clk)
      if(reset)
         y <= IDLE;
      else
         y <= Y;   
   
   // memory read data valid delay counter
   always@(posedge clk) begin
      if(y == IDLE)
         mem_delay_ctr <= 2'b0;
      else if(y == MEMREAD)
         mem_delay_ctr <= mem_delay_ctr + 1;
   end

   // memory address counter for 640 x 480
   always@(posedge clk) begin
      if(reset) begin
         mem_addr <= 19'hE; //skip bmp header
         last_data <= 1'b0;
      end
      else if(mem_addr == 19'h4B00E) begin //19'h4B00E
         mem_addr <= 19'hE; //bring ctr to start
         last_data <= 1'b1;
      end
      else if(mem_addr_ctr_en) begin
         mem_addr <= mem_addr + 1;
         last_data <= 1'b0;
      end
      //adjust the counter b/c counter overincrements before FIFO full signal is received
      else if(full && (y == EN_WR2FIFO)) begin
         mem_addr <= mem_addr - 5;
         last_data <= 1'b0;
      end
   end  
endmodule